import shlex
from subprocess import Popen


class CommandRunner:
    def __init__(self, shell: bool = False, print_cmd: bool = True,
                 exit_non_zero: bool = True, print_cmd_sep: bool = True) -> None:
        self.shell = shell
        self.print_cmd = print_cmd
        self.exit_non_zero = exit_non_zero
        self.print_cmd_sep = print_cmd_sep

    def run(self, cmd: str) -> int:
        if self.print_cmd:
            print('=' * 80)

        if self.print_cmd:
            print(f'[CMD]: {cmd}')

        p = Popen(args=shlex.split(cmd), shell=self.shell)
        p.wait()

        if self.print_cmd:
            print('=' * 80)

        if self.exit_non_zero and p.returncode != 0:
            print(
                f'Last command exited with non zero exit code. [{p.returncode}]')
            exit(1)

        return p.returncode

    def set_exit_non_zero(self, val: bool) -> None:
        self.exit_non_zero = val



import argparse


def _get_buildme_file_contents(filepath: str) -> str:
    with open(filepath, 'r', encoding='utf-8') as f:
        return f.read()


def _create_build_script_code(old_code: str, targets: list[str], usr_opt_var_name: str) -> str:
    return old_code + ''.join(f'\n{t}({usr_opt_var_name})' for t in targets)


def main() -> int:
    parser = argparse.ArgumentParser()
    user_arg_parser = argparse.ArgumentParser()

    parser.add_argument('path', type=str, help='path to the buildme script')
    parser.add_argument('targets', type=str, nargs='*',
                        help='The list of targets to run')

    args, unknown = parser.parse_known_args()

    for a in unknown:
        if a.startswith(('-', '--')):
            user_arg_parser.add_argument(a.split('=')[0], type=str)

    usr_known_args, _ = user_arg_parser.parse_known_args()
    usr_known_args_var_name = f'{usr_known_args=}'.split(
        '=')[0]  # gets the var name as string

    new_buildme_code = _create_build_script_code(
        _get_buildme_file_contents(args.path),
        args.targets,
        usr_known_args_var_name
    )

    # the dangerous part
    exec(new_buildme_code, {
         usr_known_args_var_name: usr_known_args, **globals()})

    return 0


if __name__ == '__main__':
    raise SystemExit(main())



# helper funcs
from pathlib import Path


def mkdir(path: str) -> None:
    Path(path).mkdir(exist_ok=True, parents=True)


def touch(filepath: str) -> None:
    Path(filepath).touch()


create_file = touch
